
#	irc-bot-js-webcompat
#	--------------------
#	Query web standards browser support by feature
#
#	Provides one bot command: 'compat'

'use strict'


util    = require 'util'
fs      = require 'fs'
shorten = require 'goo.gl'



# Load the dataset
dataset = do ->
	dataset = fs.readFileSync "#{__dirname}/../data.json", 'utf-8'
	return JSON.parse dataset if dataset



module.exports = ->

	config = @bot_options.plugin_options?['irc-js-bot-webcompat'] or {}
	config.browsers or= []



	#
	#	Command: compat
	#

	@register_special_command
		name: 'compat'
		description: 'Query web standards browser support by feature'
		admin_only: false
		fn: (event, input_data, output_data) =>

			if '?' in input_data.flags
				message = [
					'''Syntax: compat[/ais] <feature> • Query web standards browser support by feature'''
					'''--- This command supports the following flags ---'''
					'''a • all browsers: show compatibility for all browsers'''
					'''i • info: show additional information for a feature (description, useful links, …)'''
					'''s • search: find a feature's internal name for use with the main command'''
				]
				@send 'notice', event.person.nick, message
				return

			# Search feature names
			else if 's' in input_data.flags
				query = input_data.args.trim()
				return if not query

				results = []

				for feature of dataset.data
					results.push feature if (RegExp query, 'i').test feature

				if results.length
					message = "Found #{results.length} results for query « #{query} » #{results.join ' • '}"
				else
					message = "Sorry, no results found for query « #{query} »"

				@send 'notice', event.person.nick, message
				return

			# Display information regarding the feature (description, status, notes, spec link)
			else if 'i' in input_data.flags
				feature = input_data.args.trim()

				if not (feature of dataset.data)
					message = "Sorry, « #{feature} » is not a feature tracked by the dataset"
				else
					feature_data = dataset.data[feature]
					notes = " • #{feature_data.notes}" or ''

					message = "« #{feature_data.title} • #{feature_data.description} » Status: #{dataset.statuses[feature_data.status]} • Spec: #{feature_data.spec}#{notes}"

			# Display feature stats
			else
				feature = input_data.args.trim()

				if not (feature of dataset.data)
					message = "Sorry, « #{feature} » is not a feature tracked by the dataset"
				else
					support = []
					feature_data = dataset.data[feature]

					for own browser_name, browser_data of feature_data.stats
						continue if (not (browser_name in config.browsers)) and (not ('a' in input_data.flags))

						# Get a list of ordered version numbers
						ordered_versions = do ->
							versions = Object.keys browser_data
							versions = versions.sort (a, b) ->
								[a, b] = [a, b].map (item) -> (item.split '-')[0] # We only care about the start of the range
								return a - b
							return versions

						for version in ordered_versions
							status = browser_data[version].split ' '
							if 'y' in status or 'a' in status
								support.push "#{dataset.agents[browser_name].browser} #{(version.split '-')[0]}"
								break

					await shorten feature_data.spec, defer shorten_err, feature_spec_uri
					await shorten "http://caniuse.com#feat=#{feature}", defer shorten_err, details_uri

					usage_pct = (feature_data.usage_perc_y + feature_data.usage_perc_a).toFixed 2

					if 'a' in input_data.flags
						message = "« #{feature_data.title} » #{support.join ' • '}"
					else
						message = "« #{feature_data.title} » #{support.join ' • '} • Global support: #{usage_pct}% (of which #{feature_data.usage_perc_a}% is partial support) • Spec: #{feature_spec_uri} • Details: #{details_uri}"

			@send output_data.method, output_data.recipient, message
